#include <PCF8574.h>

/* PCF8574P - Para os Veículos
 * Porta 0 - Vermelho - Via 1
 * Porta 1 - Verde - Via 2
 * Porta 2 - Amarelo Prudencia para virar à esquerda - Via 2
 * Porta 3 - Amarelo Prudencia para virar à esquerda - Via 1
 * Porta 4 - Vermelho - Via 2
 * Porta 5 - Amarelo - Via 2
 * Porta 6 - Verde - Via 1
 * Porta 7 - Amarelo  - Via 1 * 
 */

PCF8574 pcf_Veiculos(0x38); // Endereço do PCF8574P
  
byte displaySeteSeg[10][7] = { // array com todos os digitos de zero a nove  
 { 1,1,1,1,1,1,0 },  //DIGITO 0
 { 0,1,1,0,0,0,0 },  //DIGITO 1
 { 1,1,0,1,1,0,1 },  //DIGITO 2
 { 1,1,1,1,0,0,1 },  //DIGITO 3
 { 0,1,1,0,0,1,1 },  //DIGITO 4
 { 1,0,1,1,0,1,1 },  //DIGITO 5
 { 1,0,1,1,1,1,1 },  //DIGITO 6
 { 1,1,1,0,0,0,0 },  //DIGITO 7
 { 1,1,1,1,1,1,1 },  //DIGITO 8
 { 1,1,1,1,0,1,1 }   //DIGITO 9 
};

/* PCF8574P - Para os semáforos Pedonais */
PCF8574 pcf_Display1(0x39); // 0x39 Endereço do PCF8574P
PCF8574 pcf_Display2(0x3D); // 0x3D Endereço do PCF8574P

 // Portas a usar pelos LEDs dos semáforos pedonais
const int LedR1 = 7;
const int LedG1 = 6;
const int LedR2 = 5;
const int LedG2 = 4;

int temp = 250;   // Tempo dado ao sinal intermitente - Sem efeito por causa do display 
const int tempoDisplay = 9; // Número de segundos para passar a passadeira 

/* Luz de Rua */ 
const int btLigaDesliga = 8;
const int luz           = 9;
bool ligada             = false;
 
void setup() {
  Serial.begin(9600);  // Inicia o monitor de serie para monitorizar o circuito
  for(int i=0; i<=7; i++){ // pinMode das portas dos PCF8574P Veículos + Semáforos
    pcf_Veiculos.pinMode(i, OUTPUT);
    pcf_Display1.pinMode(i, OUTPUT);
    pcf_Display2.pinMode(i, OUTPUT);
  }
  pcf_Veiculos.begin();  
  pcf_Display1.begin(); 
  pcf_Display2.begin();

  for(int i=4; i<=7; i++){ // pinMode das portas dos semáforos pedonais
    pinMode(i, OUTPUT);
  }

  pinMode(btLigaDesliga, INPUT);  // pinMode da porta do botão da luz de Rua
  pinMode(luz, OUTPUT);           // pinMode da porta da luz de Rua

  /* Inicia o sistema */
  //Inicializar semáforos para Veículos
  for(int i=0; i<=7; i++){ // garantir que todos os semáforos estão desligados
    pcf_Veiculos.digitalWrite(i, LOW);
  }
  
  // Inicializar semáforos pedonais
  for(int i=4; i<=7; i++){
    digitalWrite(i, LOW);
  }
  ligaSegmentosDisplay1(0); // Coloca o display 1 com o valor zero
  ligaSegmentosDisplay2(0); // Coloca o display 2 com o valor zero
}

void loop() {  
  // Ligar semáforos 
  // ligar os semáforos para os veículos para a primeira Via
  pcf_Veiculos.digitalWrite(0, HIGH); // Veiculos - Liga LED Vermelho Via 1
  pcf_Veiculos.digitalWrite(1, HIGH); // Veiculos - Liga LED Verde Via 2
  pcf_Veiculos.digitalWrite(2, HIGH); // Veiculos - Liga LED Amarelo Prudência Via 2

  digitalWrite(LedR1, LOW);   // Pedonal - Desliga LED Vermelho Via 1
  digitalWrite(LedG1, HIGH);  // Pedonal - Liga LED Verde Via 1  
  digitalWrite(LedR2, HIGH);  // Pedonal - Liga LED Vermelho Via 2
  digitalWrite(LedG2, LOW);   // Pedonal - Desliga LED Verde Via 2
  
  for(int i=tempoDisplay; i>=0; i--){ // Display vai fazer a contagem decrescente 
    ligaSegmentosDisplay1(i);
    if(i>3){
      delay(1000);
    }else{
      // Desliga o semáforo verde e liga o amarelo intermitente  
      pcf_Veiculos.digitalWrite(1, LOW); // Veiculos - Desliga LED Verde Via 2
      for(int j=0; j<=1; j++){ // O LED Amarelo vai piscar duas vezes o que equivale a 1 segundo, não pode ser mais por causa do display
        pcf_Veiculos.digitalWrite(5, HIGH);
        delay(temp);
        pcf_Veiculos.digitalWrite(5, LOW);
        delay(temp);    
      } 
    } 
    leBotao(); // Lê o valor do botão a cada um segundo;  
  }
  
  // Termina o primeiro ciclo, vamos passar à segunda via
  // Desliga os LEDs Amarelo e Vermelho e Liga o Vermelho e o verde e Amarelo do semáforo oposto
  pcf_Veiculos.digitalWrite(0, LOW);  // Veiculos - Desliga LED Vermelho Via 1
  pcf_Veiculos.digitalWrite(2, LOW);  // Veiculos - Desliga LED Amarelo Prudência Via 2
  pcf_Veiculos.digitalWrite(4, HIGH); // Veiculos - Liga LED Vermelho Via 2
  pcf_Veiculos.digitalWrite(6, HIGH); // Veiculos - Liga LED Verde Via 1
  pcf_Veiculos.digitalWrite(3, HIGH); // Veiculos - Liga LED Amarelo Prudência Via 1
  
  digitalWrite(LedR1, HIGH);  // Pedonal - Liga LED Vermelho Via 1
  digitalWrite(LedG1, LOW);   // Pedonal - Desliga LED Verde Via 1  
  digitalWrite(LedR2, LOW);   // Pedonal - Desliga LED Vermelho Via 2
  digitalWrite(LedG2, HIGH);  // Pedonal - Liga LED Verde Via 2
  
  for(int i=tempoDisplay; i>=0; i--){ // Display vai fazer a contagem decrescente 
    ligaSegmentosDisplay2(i);
    if(i>3){
      delay(1000);
    }else{
      // Desliga o semáforo verde e liga o amarelo intermitente  
      pcf_Veiculos.digitalWrite(6, LOW); // Veiculos - Desliga LED Verde Via 1
      for(int j=0; j<=1; j++){ // O LED Amarelo vai piscar duas vezes o que equivale a 1 segundo, não pode ser mais por causa do display
        pcf_Veiculos.digitalWrite(7, HIGH);
        delay(temp);
        pcf_Veiculos.digitalWrite(7, LOW);
        delay(temp);    
      } 
    } 
    leBotao(); // Lê o valor do botão a cada um segundo;  
  }
  
  // Termina o segundo ciclo, vamos desligar os semáforos para voltar ao primeiro ciclo
  pcf_Veiculos.digitalWrite(4, LOW); // Veiculos - Desliga LED Vermelho Via 2
  pcf_Veiculos.digitalWrite(3, LOW); // Veiculos - Desliga LED Amarelo Prudência Via 1
}

void ligaSegmentosDisplay1(byte digito){ //função para acionar o display com o digito correspondente   
  byte pino = 0;
   
  //Liga os segmentos de acordo com o digito correspondente
  for (byte contadorSegmentos = 0; contadorSegmentos < 7; ++contadorSegmentos){ 
    pcf_Display1.digitalWrite(pino, displaySeteSeg[digito][contadorSegmentos]);
    Serial.print(pino);
    Serial.print(" - ");
    Serial.print(digito);
    Serial.print(" - ");
    Serial.println(contadorSegmentos);
    pino++; 
  }
}

void ligaSegmentosDisplay2(byte digito){ //função para acionar o display com o digito correspondente   
  byte pino = 0;
   
  //Liga os segmentos de acordo com o digito correspondente
  for (byte contadorSegmentos = 0; contadorSegmentos < 7; ++contadorSegmentos){ 
    pcf_Display2.digitalWrite(pino, displaySeteSeg[digito][contadorSegmentos]);
    Serial.print(pino);
    Serial.print(" - ");
    Serial.print(digito);
    Serial.print(" - ");
    Serial.println(contadorSegmentos);
    pino++; 
  }
}

void leBotao(){
  int value = digitalRead(btLigaDesliga);
  Serial.println(value);

  if(value == 1){
    if(!ligada){ // Liga o LED
      Serial.println("Liga a Luz");
      digitalWrite(luz, HIGH);
      ligada = true;
    } else { // Desliga o LED
      Serial.println("Desliga a Luz");
      digitalWrite(luz, LOW);
      ligada = false;
    }
  }
}
